/* OBSOLETE 	.text */
/* OBSOLETE 	.global	_convert_from_extended */
/* OBSOLETE  */
/* OBSOLETE _convert_from_extended: */
/* OBSOLETE  */
/* OBSOLETE 	ldfe	f0,[a1] */
/* OBSOLETE 	stfd	f0,[a2] */
/* OBSOLETE 	movs	pc,lr */
/* OBSOLETE  */
/* OBSOLETE 	.global	_convert_to_extended */
/* OBSOLETE  */
/* OBSOLETE _convert_to_extended: */
/* OBSOLETE  */
/* OBSOLETE 	ldfd	f0,[a1] */
/* OBSOLETE 	stfe	f0,[a2] */
/* OBSOLETE 	movs	pc,lr */
